@@ -637,7 +637,7 @@ update_directory (BusActivation       *activation,
 
       if (!_dbus_string_ends_with_c_str (&filename, ".service"))
         {
-          _dbus_verbose ("Skipping non-.service file %s\n",
+          _dbus_verbose ("Skipping non-.service file '%s'\n",
                          _dbus_string_get_const_data (&filename));
           continue;
         }
@@ -1044,7 +1044,7 @@ restore_pending (void *data)
 }
 
 static void
-free_pending_restore_data (void *data)
+free_restore_pending_data (void *data)
 {
   RestorePendingData *d = data;
 
@@ -1074,9 +1074,9 @@ add_restore_pending_to_transaction (BusTransaction       *transaction,
 
   if (d->hash_entry == NULL ||
       !bus_transaction_add_cancel_hook (transaction, restore_pending, d,
-                                        free_pending_restore_data))
+                                        free_restore_pending_data))
     {
-      free_pending_restore_data (d);
+      free_restore_pending_data (d);
       return FALSE;
     }
 
@@ -1478,11 +1478,8 @@ pending_activation_timed_out (void *data)
 
   dbus_error_init (&error);
 
-  dbus_set_error (&error, DBUS_ERROR_TIMED_OUT,
-                  "Activation of %s timed out",
-                  pending_activation->service_name);
-  bus_context_log (pending_activation->activation->context,
-                   DBUS_SYSTEM_LOG_INFO,
+  bus_context_log_and_set_error (pending_activation->activation->context,
+                   DBUS_SYSTEM_LOG_INFO, &error, DBUS_ERROR_TIMED_OUT,
                    "Failed to activate service '%s': timed out",
                    pending_activation->service_name);
 
@@ -1704,7 +1701,7 @@ child_setup (void *user_data)
     {
       /* unfortunately we don't actually know the service name here */
       bus_context_log (activation->context,
-                       DBUS_SYSTEM_LOG_INFO,
+                       DBUS_SYSTEM_LOG_WARNING,
                        "Failed to reset fd limit before activating "
                        "service: %s: %s",
                        error.name, error.message);
@@ -2007,6 +2004,17 @@ bus_activation_activate_service (BusActivation  *activation,
           _dbus_string_init_const (&service_string, "org.freedesktop.systemd1");
           service = bus_registry_lookup (registry, &service_string);
 
+          /* Following the general principle of "log early and often",
+           * we capture that we *want* to send the activation message, even if
+           * systemd is not actually there to receive it yet */
+          if (!bus_transaction_capture (activation_transaction,
+                NULL, message))
+            {
+              dbus_message_unref (message);
+              BUS_SET_OOM (error);
+              return FALSE;
+            }
+
           if (service != NULL)
             {
               bus_context_log (activation->context,
@@ -2249,7 +2257,7 @@ dbus_activation_systemd_failure (BusActivation *activation,
                              DBUS_TYPE_STRING, &code,
                              DBUS_TYPE_STRING, &str,
                              DBUS_TYPE_INVALID))
-    dbus_set_error (&error, code, "%s", str);
+    dbus_set_error(&error, code, str);
 
 
   if (unit)
